package cz.muni.fi.pb162.test3;

import java.text.DateFormat;
import java.util.Date;
import junit.framework.TestCase;

public class AirportTest extends TestCase {
    
    public AirportTest(String testName) {
        super(testName);
    }

    Airplane ok182 = new AirplaneImpl(new String("OK 182"), 1000);
    Airplane ok182b = new AirplaneImpl(new String("OK 182"), 2000);
    Airplane ok184 = new AirplaneImpl("OK 184", 1000);
    Airplane ok171 = new AirplaneImpl("OK 171", 500);

    public void testAirplaneGetId() {
        assertEquals("OK 182",ok182.getID());
        assertEquals("OK 184",ok184.getID());
    }

    public void testAirplaneGetTimeOfRunningOutOfOil() {
        assertEquals(1000,ok182.getTimeOfRunningOutOfOil());
        assertEquals(500,ok171.getTimeOfRunningOutOfOil());
    }

    public void testAirplaneEquals() {
        Object o = ok182;
        assertTrue(o.equals(ok182b));
        assertFalse(o.equals(""));
        assertFalse(o.equals(null));
        assertFalse(o.equals(ok184));
    }

    public void testAirplaneHashcode() {
        assertTrue(ok182.hashCode() == ok182b.hashCode());
    }

    public void testAirplaneToString() {
        DateFormat dateFormat = DateFormat.getDateTimeInstance();
        assertEquals("Letadlo OK 182, palivo dojde " +
                dateFormat.format(new Date(1000)), ok182.toString());
        assertEquals("Letadlo OK 184, palivo dojde " +
                dateFormat.format(new Date(1000)), ok184.toString());
        assertEquals("Letadlo OK 171, palivo dojde " +
                dateFormat.format(new Date(500)), ok171.toString());
    }

    public void testNewAirplane() {
        try {
            new AirplaneImpl(null,0);
            fail("Konstruktor AirplaneImpl nevyhazuje NullPointerException, " +
                    "kdyz se nekdo pokusi vytvorit instanci s id = null");
        } catch(NullPointerException ex) {
            // All is ok
        }
    }

    public void testApproach() throws AirplaneCrashException {
        Airport airport = new AirportImpl();
        TimeService.setCurrentTime(1000);

        // Tohle by se jeste melo povest
        airport.approach(new AirplaneImpl("x",1000));
        // A tohle uz ne...
        try {
            airport.approach(new AirplaneImpl("x",999));
            fail();
        } catch(AirplaneCrashException ex) {
            // All is ok
        }
    }

    public void testLand() throws AirplaneCrashException {
        TimeService.setCurrentTime(0);
        Airport airport = new AirportImpl();
        assertNull(airport.land());
        Airplane a10 = new AirplaneImpl("10",10);
        Airplane a10000 = new AirplaneImpl("10000",100000);
        Airplane a5 = new AirplaneImpl("5",5);
        airport.approach(a10);
        airport.approach(a10000);
        airport.approach(a5);
        assertEquals(a5,airport.land());
        assertEquals(a10,airport.land());
        assertEquals(a10000,airport.land());
        assertNull(airport.land());
        assertNull(airport.land());
        assertNull(airport.land());
    }
    
    public void testLandCrash() throws AirplaneCrashException {
        TimeService.setCurrentTime(0);
        Airport airport = new AirportImpl();
        Airplane a10 = new AirplaneImpl("10",10);
        Airplane a11 = new AirplaneImpl("11",11);
        Airplane a10000 = new AirplaneImpl("10000",100000);
        airport.approach(a11);
        airport.approach(a10000);
        airport.approach(a10);

        TimeService.setCurrentTime(10);
        // Tohle by se jeste melo povest
        assertEquals(a10,airport.land());
        TimeService.setCurrentTime(12);
        // Tohle uz ne
        try {
            airport.land();
            fail();
        } catch(AirplaneCrashException ex) {
            // All is ok
        }
        assertEquals(a10000,airport.land());
        assertNull(airport.land());
    }
    
}
